==> pylint_ignore-2020.1005-py2.py3-none-any.whl: pylint_ignore/__main__.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the pylint-ignore project
# https://gitlab.com/mbarkhau/pylint-ignore
#
# Copyright (c) 2020 Manuel Barkhau (mbarkhau@gmail.com) - MIT License
# SPDX-License-Identifier: MIT
"""CLI for pylint-ignore.

This module wraps the pylint runner and supresses individual
messages if configured via a pylint-ignore.md file.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import os
import re
import sys
import typing as typ
import getpass
import logging
import datetime as dt
import subprocess as sp
import pathlib2 as pl
try:
    import builtins
except ImportError:
    import __builtin__ as builtins
import pylint.lint
str = getattr(builtins, 'unicode', str)
from . import ignorefile
try:
    from pylint.message.message_handler_mix_in import MessagesHandlerMixIn
except ImportError:
    from pylint.utils import MessagesHandlerMixIn


def _pylint_msg_defs(linter, msgid):
    if hasattr(linter.msgs_store, 'get_message_definitions'):
        return linter.msgs_store.get_message_definitions(msgid)
    else:
        return [linter.msgs_store.get_message_definition(msgid)]


if os.environ.get('ENABLE_RICH_TB') == '1':
    try:
        import rich.traceback
        rich.traceback.install()
    except ImportError:
        pass
logger = logging.getLogger('pylint_ignore')
ExitCode = int
USAGE_ERROR = 32
MaybeLineNo = typ.Optional[int]
DEFAULT_IGNOREFILE_PATH = pl.Path('.') / 'pylint-ignore.md'


def _run(cmd):
    cmd_parts = cmd.split()
    try:
        output = sp.check_output(cmd_parts)
    except FileNotFoundError:
        return ''
    except sp.SubprocessError:
        return ''
    return output.strip().decode('utf-8')


def get_author_name():
    """Do a best effort to get a meaningful author name."""
    hg_username = _run('hg config ui.username')
    git_email = _run('git config user.email')
    git_name = _run('git config user.name')
    if git_email and '<' in git_email and '>' in git_email:
        git_username = git_email
    elif git_name and git_email:
        git_username = git_name + ' <' + git_email + '>'
    elif git_name:
        git_username = git_name
    elif git_email:
        git_username = git_email
    else:
        git_username = ''
    is_hg_repo = pl.Path('.hg').exists()
    is_git_repo = pl.Path('.git').exists()
    if is_hg_repo and hg_username:
        return hg_username
    if is_git_repo and git_username:
        return git_username
    if hg_username:
        return hg_username
    if git_username:
        return git_username
    return getpass.getuser()


class PylintIgnoreDecorator(object):
    ignorefile_path = None
    is_update_mode = None
    pylint_run_args = None
    default_author = None
    default_date = None
    old_catalog = None
    new_catalog = None
    _pylint_is_message_enabled = None
    _pylint_add_message = None
    _cur_msg_args = None

    def __init__(self, args):
        self.ignorefile_path = DEFAULT_IGNOREFILE_PATH
        self.is_update_mode = False
        self.pylint_run_args = []
        self._init_from_args(args)
        self.old_catalog = ignorefile.load(self.ignorefile_path)
        self.new_catalog = {}
        self.default_author = get_author_name()
        self.default_date = dt.datetime.now().isoformat().split('.')[0]
        self._cur_msg_args = []

    def _init_from_args(self, args):
        arg_i = 0
        while arg_i < len(args):
            arg = args[arg_i]
            if arg == '--update-ignorefile':
                self.is_update_mode = True
            elif arg == '--ignorefile':
                self.ignorefile_path = pl.Path(args[arg_i + 1])
            elif arg.startswith('--ignorefile='):
                self.ignorefile_path = pl.Path(arg.split('=', 1)[-1])
            elif arg.startswith('--jobs') or arg.startswith('-j'):
                if '=' in arg:
                    num_jobs = arg.split('=', 1)[-1]
                else:
                    num_jobs = args[arg_i + 1]
                if num_jobs != '1':
                    if '=' in arg:
                        sys.stderr.write('Invalid argument {0}\n'.format(arg))
                    else:
                        sys.stderr.write('Invalid argument {0} {1}\n'.
                            format(arg, num_jobs))
                    sys.stderr.write(
                        '    pylint-ignore only works with --jobs=1\n')
                    raise SystemExit(USAGE_ERROR)
            else:
                self.pylint_run_args.append(arg)
            arg_i += 1
        if not self.ignorefile_path.exists() and not self.is_update_mode:
            sys.stderr.write('Invalid path, does not exist: {0}\n'.format(
                self.ignorefile_path))
            raise SystemExit(USAGE_ERROR)
        self.pylint_run_args.insert(0, '--jobs=1')

    def _new_entry(self, key, old_entry, msg_text, srctxt):
        if old_entry:
            author = old_entry.author
            date = old_entry.date
        else:
            author = self.default_author
            date = self.default_date
        return ignorefile.Entry(key.msg_id, key.path, key.symbol, msg_text,
            author, date, srctxt)

    def is_enabled_entry(self, msg_id, path, symbol, msg_text, srctxt):
        """Return false if message is in the serialized catalog.

        Side effect: Track new entries for serialization.
        """
        pwd = pl.Path('.').absolute()
        rel_path = str(pl.Path(path).absolute().relative_to(pwd))
        source_line = srctxt.source_line if srctxt else ''
        key = ignorefile.Key(msg_id, rel_path, symbol, msg_text, source_line)
        old_entry = ignorefile.find_entry(self.old_catalog, key)
        new_entry = self._new_entry(key, old_entry, msg_text, srctxt)
        self.new_catalog[key] = new_entry
        is_ignored = old_entry is not None or self.is_update_mode
        return not is_ignored

    def _is_message_enabled_wrapper(self):

        def is_any_message_def_enabled(linter, msgid, line):
            srctxt = ignorefile.read_source_text(linter.current_file, line,
                line) if line else None
            for msg_def in _pylint_msg_defs(linter, msgid):
                if len(self._cur_msg_args) >= msg_def.msg.count('%'):
                    msg_text = msg_def.msg % tuple(self._cur_msg_args)
                else:
                    msg_text = msg_def.msg
                _is_enabled = self.is_enabled_entry(msgid, linter.
                    current_file, msg_def.symbol, msg_text, srctxt)
                if not _is_enabled:
                    return False
            return True

        def is_message_enabled(linter, msg_descr, line=None, confidence=None):
            is_enabled = self._pylint_is_message_enabled(linter, msg_descr,
                line, confidence)
            if not is_enabled:
                return False
            if re.match('\\w\\d{1,5}', msg_descr) is None:
                return True
            if linter.current_file is None:
                return True
            try:
                return is_any_message_def_enabled(linter, msg_descr, line)
            finally:
                del self._cur_msg_args[:]
        return is_message_enabled

    def _add_message_wrapper(self):

        def add_message(linter, msgid, line=None, node=None, args=None,
            confidence=None, col_offset=None):
            del self._cur_msg_args[:]
            if isinstance(args, tuple):
                self._cur_msg_args.extend(args)
            elif isinstance(args, (bytes, str)):
                self._cur_msg_args.append(args)
            self._pylint_add_message(linter, msgid, line, node, args,
                confidence, col_offset)
        return add_message

    def monkey_patch_pylint(self):
        self._pylint_is_message_enabled = (MessagesHandlerMixIn.
            is_message_enabled)
        self._pylint_add_message = MessagesHandlerMixIn.add_message
        MessagesHandlerMixIn.is_message_enabled = (self.
            _is_message_enabled_wrapper())
        MessagesHandlerMixIn.add_message = self._add_message_wrapper()

    def monkey_unpatch_pylint(self):
        MessagesHandlerMixIn.is_message_enabled = (self.
            _pylint_is_message_enabled)
        MessagesHandlerMixIn.add_message = self._pylint_add_message


def main(args=sys.argv[1:]):
    dec = PylintIgnoreDecorator(args)
    try:
        dec.monkey_patch_pylint()
        exit_code = 0
        try:
            pylint.lint.Run(dec.pylint_run_args)
        except SystemExit as sysexit:
            exit_code = sysexit.code
        except KeyboardInterrupt:
            return 1
        is_catalog_dirty = dec.old_catalog != dec.new_catalog
        if is_catalog_dirty and dec.is_update_mode:
            ignorefile.dump(dec.new_catalog, dec.ignorefile_path)
    finally:
        dec.monkey_unpatch_pylint()
    return exit_code


if __name__ == '__main__':
    sys.exit(main())

==> pylint_ignore-2020.1006-py2.py3-none-any.whl: pylint_ignore/__main__.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the pylint-ignore project
# https://gitlab.com/mbarkhau/pylint-ignore
#
# Copyright (c) 2020 Manuel Barkhau (mbarkhau@gmail.com) - MIT License
# SPDX-License-Identifier: MIT
"""CLI for pylint-ignore.

This module wraps the pylint runner and supresses individual
messages if configured via a pylint-ignore.md file.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import os
import re
import sys
import typing as typ
import getpass
import logging
import datetime as dt
import functools as ft
import subprocess as sp
import pathlib2 as pl
try:
    import builtins
except ImportError:
    import __builtin__ as builtins
import pylint.lint
str = getattr(builtins, 'unicode', str)
from . import ignorefile
try:
    from pylint.message.message_handler_mix_in import MessagesHandlerMixIn
except ImportError:
    from pylint.utils import MessagesHandlerMixIn


def _pylint_msg_defs(linter, msgid):
    if hasattr(linter.msgs_store, 'get_message_definitions'):
        return linter.msgs_store.get_message_definitions(msgid)
    else:
        return [linter.msgs_store.get_message_definition(msgid)]


if os.environ.get('ENABLE_RICH_TB') == '1':
    try:
        import rich.traceback
        rich.traceback.install()
    except ImportError:
        pass
logger = logging.getLogger('pylint_ignore')
ExitCode = int
USAGE_ERROR = 32
MaybeLineNo = typ.Optional[int]
DEFAULT_IGNOREFILE_PATH = pl.Path('.') / 'pylint-ignore.md'


def _run(cmd):
    cmd_parts = cmd.split()
    try:
        output = sp.check_output(cmd_parts)
    except FileNotFoundError:
        return ''
    except sp.SubprocessError:
        return ''
    return output.strip().decode('utf-8')


def get_author_name():
    """Do a best effort to get a meaningful author name."""
    hg_username = _run('hg config ui.username')
    git_email = _run('git config user.email')
    git_name = _run('git config user.name')
    if git_email and '<' in git_email and '>' in git_email:
        git_username = git_email
    elif git_name and git_email:
        git_username = git_name + ' <' + git_email + '>'
    elif git_name:
        git_username = git_name
    elif git_email:
        git_username = git_email
    else:
        git_username = ''
    is_hg_repo = pl.Path('.hg').exists()
    is_git_repo = pl.Path('.git').exists()
    if is_hg_repo and hg_username:
        return hg_username
    if is_git_repo and git_username:
        return git_username
    if hg_username:
        return hg_username
    if git_username:
        return git_username
    return getpass.getuser()


class PylintIgnoreDecorator(object):
    ignorefile_path = None
    is_update_mode = None
    pylint_run_args = None
    default_author = None
    default_date = None
    old_catalog = None
    new_catalog = None
    _pylint_is_message_enabled = None
    _pylint_add_message = None
    _cur_msg_args = None

    def __init__(self, args):
        self.ignorefile_path = DEFAULT_IGNOREFILE_PATH
        self.is_update_mode = False
        self.pylint_run_args = []
        self._init_from_args(args)
        self.old_catalog = ignorefile.load(self.ignorefile_path)
        self.new_catalog = {}
        self.default_author = get_author_name()
        self.default_date = dt.datetime.now().isoformat().split('.')[0]
        self._cur_msg_args = []

    def _init_from_args(self, args):
        arg_i = 0
        while arg_i < len(args):
            arg = args[arg_i]
            if arg == '--update-ignorefile':
                self.is_update_mode = True
            elif arg == '--ignorefile':
                self.ignorefile_path = pl.Path(args[arg_i + 1])
            elif arg.startswith('--ignorefile='):
                self.ignorefile_path = pl.Path(arg.split('=', 1)[-1])
            elif arg.startswith('--jobs') or arg.startswith('-j'):
                if '=' in arg:
                    num_jobs = arg.split('=', 1)[-1]
                else:
                    num_jobs = args[arg_i + 1]
                if num_jobs != '1':
                    if '=' in arg:
                        sys.stderr.write('Invalid argument {0}\n'.format(arg))
                    else:
                        sys.stderr.write('Invalid argument {0} {1}\n'.
                            format(arg, num_jobs))
                    sys.stderr.write(
                        '    pylint-ignore only works with --jobs=1\n')
                    raise SystemExit(USAGE_ERROR)
            else:
                self.pylint_run_args.append(arg)
            arg_i += 1
        if not self.ignorefile_path.exists() and not self.is_update_mode:
            sys.stderr.write('Invalid path, does not exist: {0}\n'.format(
                self.ignorefile_path))
            raise SystemExit(USAGE_ERROR)
        self.pylint_run_args.insert(0, '--jobs=1')

    def _new_entry(self, key, old_entry, msg_text, srctxt):
        if old_entry:
            author = old_entry.author
            date = old_entry.date
        else:
            author = self.default_author
            date = self.default_date
        return ignorefile.Entry(key.msg_id, key.path, key.symbol, msg_text,
            author, date, srctxt)

    def is_enabled_entry(self, msg_id, path, symbol, msg_text, srctxt):
        """Return false if message is in the serialized catalog.

        Side effect: Track new entries for serialization.
        """
        pwd = pl.Path('.').absolute()
        rel_path = str(pl.Path(path).absolute().relative_to(pwd))
        source_line = srctxt.source_line if srctxt else ''
        key = ignorefile.Key(msg_id, rel_path, symbol, msg_text, source_line)
        old_entry = ignorefile.find_entry(self.old_catalog, key)
        new_entry = self._new_entry(key, old_entry, msg_text, srctxt)
        self.new_catalog[key] = new_entry
        is_ignored = old_entry is not None or self.is_update_mode
        return not is_ignored

    def _add_message_wrapper(self):

        @ft.wraps(self._pylint_add_message)
        def add_message(linter, msgid, line=None, node=None, args=None,
            confidence=None, col_offset=None):
            del self._cur_msg_args[:]
            if isinstance(args, tuple):
                self._cur_msg_args.extend(args)
            elif isinstance(args, (bytes, str)):
                self._cur_msg_args.append(args)
            self._pylint_add_message(linter, msgid, line, node, args,
                confidence, col_offset)
        return add_message

    def _is_message_enabled_wrapper(self):

        def is_any_message_def_enabled(linter, msgid, line):
            srctxt = ignorefile.read_source_text(linter.current_file, line,
                line) if line else None
            for msg_def in _pylint_msg_defs(linter, msgid):
                if len(self._cur_msg_args) >= msg_def.msg.count('%'):
                    msg_text = msg_def.msg % tuple(self._cur_msg_args)
                else:
                    msg_text = msg_def.msg
                _is_enabled = self.is_enabled_entry(msgid, linter.
                    current_file, msg_def.symbol, msg_text, srctxt)
                if not _is_enabled:
                    return False
            return True

        @ft.wraps(self._pylint_is_message_enabled)
        def is_message_enabled(linter, msg_descr, line=None, confidence=None):
            try:
                is_enabled = self._pylint_is_message_enabled(linter,
                    msg_descr, line, confidence)
                if not is_enabled:
                    return False
                is_always_enabled = re.match('\\w\\d{1,5}', msg_descr
                    ) is None or msg_descr[:1
                    ] == 'E' or linter.current_file is None
                if is_always_enabled:
                    return True
                return is_any_message_def_enabled(linter, msg_descr, line)
            finally:
                del self._cur_msg_args[:]
        return is_message_enabled

    def monkey_patch_pylint(self):
        self._pylint_is_message_enabled = (MessagesHandlerMixIn.
            is_message_enabled)
        self._pylint_add_message = MessagesHandlerMixIn.add_message
        MessagesHandlerMixIn.is_message_enabled = (self.
            _is_message_enabled_wrapper())
        MessagesHandlerMixIn.add_message = self._add_message_wrapper()

    def monkey_unpatch_pylint(self):
        MessagesHandlerMixIn.is_message_enabled = (self.
            _pylint_is_message_enabled)
        MessagesHandlerMixIn.add_message = self._pylint_add_message


def main(args=sys.argv[1:]):
    dec = PylintIgnoreDecorator(args)
    try:
        dec.monkey_patch_pylint()
        exit_code = 0
        try:
            pylint.lint.Run(dec.pylint_run_args)
        except SystemExit as sysexit:
            exit_code = sysexit.code
        except KeyboardInterrupt:
            return 1
        is_catalog_dirty = dec.old_catalog != dec.new_catalog
        if is_catalog_dirty and dec.is_update_mode:
            ignorefile.dump(dec.new_catalog, dec.ignorefile_path)
    finally:
        dec.monkey_unpatch_pylint()
    return exit_code


if __name__ == '__main__':
    sys.exit(main())
